@@ -11,7 +11,7 @@ in settings.py for arguments that you want always passed to nose.
 from __future__ import print_function, unicode_literals
 import os
 import sys
-from optparse import make_option
+from optparse import make_option, NO_DEFAULT
 from types import MethodType
 
 import django
@@ -61,7 +61,7 @@ except ImportError:
     from django.test.simple import DjangoTestSuiteRunner as DiscoverRunner
 
 
-__all__ = ['BasicNoseRunner', 'NoseTestSuiteRunner']
+__all__ = ('BasicNoseRunner', 'NoseTestSuiteRunner')
 
 
 # This is a table of Django's "manage.py test" options which
@@ -97,21 +97,21 @@ def _get_plugins_from_settings():
             dot = plug_path.rindex('.')
         except ValueError:
             raise exceptions.ImproperlyConfigured(
-                    "%s isn't a Nose plugin module" % plug_path)
+                "%s isn't a Nose plugin module" % plug_path)
         p_mod, p_classname = plug_path[:dot], plug_path[dot + 1:]
 
         try:
             mod = import_module(p_mod)
         except ImportError as e:
             raise exceptions.ImproperlyConfigured(
-                    'Error importing Nose plugin module %s: "%s"' % (p_mod, e))
+                'Error importing Nose plugin module %s: "%s"' % (p_mod, e))
 
         try:
             p_class = getattr(mod, p_classname)
         except AttributeError:
             raise exceptions.ImproperlyConfigured(
-                    'Nose plugin module "%s" does not define a "%s"' %
-                    (p_mod, p_classname))
+                'Nose plugin module "%s" does not define a "%s"' %
+                (p_mod, p_classname))
 
         yield p_class()
 
@@ -136,12 +136,13 @@ def _get_options():
 
     # Django 1.6 introduces a "--pattern" option, which is shortened into "-p"
     # do not allow "-p" to collide with nose's "--plugins" option.
-    plugins_option = [o for o in options if o.get_opt_string() == '--plugins'][0]
+    plugins_option = [
+        o for o in options if o.get_opt_string() == '--plugins'][0]
     plugins_option._short_opts.remove('-p')
 
     django_opts = [opt.dest for opt in BaseCommand.option_list] + ['version']
-    return tuple(o for o in options if o.dest not in django_opts and
-                                       o.action != 'help')
+    return tuple(
+        o for o in options if o.dest not in django_opts and o.action != 'help')
 
 
 if hasattr(BaseCommand, 'use_argparse'):
@@ -152,7 +153,12 @@ if hasattr(BaseCommand, 'use_argparse'):
         # Don't pass the following options to nosetests
         django_opts = [
             '--noinput', '--liveserver', '-p', '--pattern', '--testrunner',
-            '--settings']
+            '--settings',
+            # 1.8 arguments
+            '--keepdb', '--reverse', '--debug-sql',
+            # 1.9 arguments
+            '--parallel',
+        ]
 
         #
         # For optparse -> argparse conversion
@@ -175,18 +181,20 @@ if hasattr(BaseCommand, 'use_argparse'):
         # If optparse has a None argument, omit from call to add_argument
         _argparse_omit_if_none = (
             'action', 'nargs', 'const', 'default', 'type', 'choices',
-            'required', 'help', 'metavar', 'dest', 'callback', 'callback_args',
-            'callback_kwargs')
+            'required', 'help', 'metavar', 'dest')
 
-        # Translating callbacks is not supported, because none of the built-in
-        # plugins uses one.  If you have a plugin that uses a callback, please
-        # open a ticket or submit a working implementation.
-        _argparse_fail_if_not_none = (
+        # Always ignore these optparse arguments
+        # Django will parse without calling the callback
+        # nose will then reparse with the callback
+        _argparse_callback_options = (
             'callback', 'callback_args', 'callback_kwargs')
 
+        # Keep track of nose options with nargs=1
+        _has_nargs = set(['--verbosity'])
+
         @classmethod
         def add_arguments(cls, parser):
-            """Convert nose's optparse arguments to argparse"""
+            """Convert nose's optparse arguments to argparse."""
             super(BaseRunner, cls).add_arguments(parser)
 
             # Read optparse options for nose and plugins
@@ -227,8 +235,15 @@ if hasattr(BaseCommand, 'use_argparse'):
                 # Convert optparse attributes to argparse attributes
                 option_attrs = {}
                 for attr in option.ATTRS:
+                    # Ignore callback options
+                    if attr in cls._argparse_callback_options:
+                        continue
+
                     value = getattr(option, attr)
 
+                    if attr == 'default' and value == NO_DEFAULT:
+                        continue
+
                     # Rename options for nose's --verbosity
                     if opt_long == '--nose-verbosity':
                         if attr == 'dest':
@@ -240,17 +255,23 @@ if hasattr(BaseCommand, 'use_argparse'):
                     if attr in cls._argparse_omit_if_none and value is None:
                         continue
 
-                    # Translating callbacks is not supported
-                    if attr in cls._argparse_fail_if_not_none:
-                        assert value is None, (
-                            'argparse option %s=%s is not supported' %
-                            (attr, value))
-                        continue
-
                     # Convert type from optparse string to argparse type
                     if attr == 'type':
                         value = cls._argparse_type[value]
 
+                    # Convert action='callback' to action='store'
+                    if attr == 'action' and value == 'callback':
+                        action = 'store'
+
+                    # Keep track of nargs=1
+                    if attr == 'nargs':
+                        assert value == 1, (
+                            'argparse option nargs=%s is not supported' %
+                            value)
+                        cls._has_nargs.add(opt_long)
+                        if opt_short:
+                            cls._has_nargs.add(opt_short)
+
                     # Pass converted attribute to optparse option
                     option_attrs[attr] = value
 
@@ -266,21 +287,36 @@ else:
         options = _get_options()
 
         # Not add following options to nosetests
-        django_opts = ['--noinput', '--liveserver', '-p', '--pattern',
-            '--testrunner']
+        django_opts = [
+            '--noinput', '--liveserver', '-p', '--pattern', '--testrunner']
+
+        # Default nosetest options with an argument
+        _has_nargs = set([
+            '--attr', '--config', '--cover-html-dir', '--cover-min-percentage',
+            '--cover-package', '--cover-xml-file', '--debug', '--debug-log',
+            '--doctest-extension', '--doctest-fixtures', '--doctest-options',
+            '--doctest-result-variable', '--eval-attr', '--exclude',
+            '--id-file', '--ignore-files', '--include', '--logging-config',
+            '--logging-datefmt', '--logging-filter', '--logging-format',
+            '--logging-level', '--match', '--process-timeout', '--processes',
+            '--profile-restrict', '--profile-sort', '--profile-stats-file',
+            '--py3where', '--tests', u'--verbosity', '--where', '--xunit-file',
+            '--xunit-testsuite-name', '-A', '-I', '-a', '-c', '-e', '-i', '-l',
+            '-m', '-w'])
 
 
 class BasicNoseRunner(BaseRunner):
-    """Facade that implements a nose runner in the guise of a Django runner
+    """Facade that implements a nose runner in the guise of a Django runner.
 
     You shouldn't have to use this directly unless the additions made by
     ``NoseTestSuiteRunner`` really bother you. They shouldn't, because they're
     all off by default.
-
     """
+
     __test__ = False
 
     def run_suite(self, nose_argv):
+        """Run the test suite."""
         result_plugin = ResultPlugin()
         plugins_to_add = [DjangoSetUpPlugin(self),
                           result_plugin,
@@ -325,7 +361,6 @@ class BasicNoseRunner(BaseRunner):
         but the extra tests will not be run.  Maybe later.
 
         Returns the number of tests that failed.
-
         """
         nose_argv = (['nosetests'] + list(test_labels))
         if hasattr(settings, 'NOSE_ARGS'):
@@ -337,9 +372,25 @@ class BasicNoseRunner(BaseRunner):
             django_opts.extend(opt._long_opts)
             django_opts.extend(opt._short_opts)
 
-        nose_argv.extend(translate_option(opt) for opt in sys.argv[1:]
-        if opt.startswith('-')
-           and not any(opt.startswith(d) for d in django_opts))
+        # Recreate the arguments in a nose-compatible format
+        arglist = sys.argv[1:]
+        has_nargs = getattr(self, '_has_nargs', set(['--verbosity']))
+        while arglist:
+            opt = arglist.pop(0)
+            if not opt.startswith('-'):
+                # Discard test labels
+                continue
+            if any(opt.startswith(d) for d in django_opts):
+                # Discard options handled by Djangp
+                continue
+
+            trans_opt = translate_option(opt)
+            nose_argv.append(trans_opt)
+
+            if opt in has_nargs:
+                # Handle arguments without an equals sign
+                opt_value = arglist.pop(0)
+                nose_argv.append(opt_value)
 
         # if --nose-verbosity was omitted, pass Django verbosity to nose
         if ('--verbosity' not in nose_argv and
@@ -358,11 +409,10 @@ _old_handle = Command.handle
 
 
 def _foreign_key_ignoring_handle(self, *fixture_labels, **options):
-    """Wrap the the stock loaddata to ignore foreign key
-    checks so we can load circular references from fixtures.
-
-    This is monkeypatched into place in setup_databases().
+    """Wrap the the stock loaddata to ignore foreign key checks.
 
+    This allows loading circular references from fixtures, and is
+    monkeypatched into place in setup_databases().
     """
     using = options.get('database', DEFAULT_DB_ALIAS)
     commit = options.get('commit', True)
@@ -383,13 +433,13 @@ def _foreign_key_ignoring_handle(self, *fixture_labels, **options):
             connection.close()
 
 
-def _skip_create_test_db(self, verbosity=1, autoclobber=False, serialize=True):
-    """``create_test_db`` implementation that skips both creation and flushing
+def _skip_create_test_db(self, verbosity=1, autoclobber=False, serialize=True,
+                         keepdb=True):
+    """``create_test_db`` implementation that skips both creation and flushing.
 
     The idea is to re-use the perfectly good test DB already created by an
     earlier test run, cutting the time spent before any tests run from 5-13s
     (depending on your I/O luck) down to 3.
-
     """
     # Notice that the DB supports transactions. Originally, this was done in
     # the method this overrides. The confirm method was added in Django v1.3
@@ -408,13 +458,12 @@ def _skip_create_test_db(self, verbosity=1, autoclobber=False, serialize=True):
 
 
 def _reusing_db():
-    """Return whether the ``REUSE_DB`` flag was passed"""
-    return os.getenv('REUSE_DB', 'false').lower() in ('true', '1', '')
+    """Return whether the ``REUSE_DB`` flag was passed."""
+    return os.getenv('REUSE_DB', 'false').lower() in ('true', '1')
 
 
 def _can_support_reuse_db(connection):
-    """Return whether it makes any sense to
-    use REUSE_DB with the backend of a connection."""
+    """Return True if REUSE_DB is a sensible option for the backend."""
     # Perhaps this is a SQLite in-memory DB. Those are created implicitly when
     # you try to connect to them, so our usual test doesn't work.
     return not connection.creation._get_test_db_name() == ':memory:'
@@ -424,7 +473,6 @@ def _should_create_database(connection):
     """Return whether we should recreate the given DB.
 
     This is true if the DB doesn't exist or the REUSE_DB env var isn't truthy.
-
     """
     # TODO: Notice when the Model classes change and return True. Worst case,
     # we can generate sqlall and hash it, though it's a bit slow (2 secs) and
@@ -437,6 +485,13 @@ def _should_create_database(connection):
 
     # Notice whether the DB exists, and create it if it doesn't:
     try:
+        # Connections are cached by some backends, if other code has connected
+        # to the database previously under a different database name the
+        # cached connection will be used and no exception will be raised.
+        # Avoiding this by closing connections and setting to null
+        for connection in connections.all():
+            connection.close()
+        connection.connection = None
         connection.cursor()
     except Exception:  # TODO: Be more discerning but still DB agnostic.
         return True
@@ -444,11 +499,10 @@ def _should_create_database(connection):
 
 
 def _mysql_reset_sequences(style, connection):
-    """Return a list of SQL statements needed to
-    reset all sequences for Django tables."""
+    """Return a SQL statements needed to reset Django tables."""
     tables = connection.introspection.django_table_names(only_existing=True)
     flush_statements = connection.ops.sql_flush(
-            style, tables, connection.introspection.sequence_list())
+        style, tables, connection.introspection.sequence_list())
 
     # connection.ops.sequence_reset_sql() is not implemented for MySQL,
     # and the base class just returns []. TODO: Implement it by pulling
@@ -461,14 +515,13 @@ def _mysql_reset_sequences(style, connection):
 
 
 class NoseTestSuiteRunner(BasicNoseRunner):
-    """A runner that optionally skips DB creation
+    """A runner that optionally skips DB creation.
 
     Monkeypatches connection.creation to let you skip creating databases if
     they already exist. Your tests will start up much faster.
 
     To opt into this behavior, set the environment variable ``REUSE_DB`` to
-    something that isn't "0" or "false" (case insensitive).
-
+    "1" or "true" (case insensitive).
     """
 
     def _get_models_for_connection(self, connection):
@@ -478,6 +531,7 @@ class NoseTestSuiteRunner(BasicNoseRunner):
                 m._meta.db_table in tables]
 
     def setup_databases(self):
+        """Setup databases, skipping DB creation if requested and possible."""
         for alias in connections:
             connection = connections[alias]
             creation = connection.creation
@@ -512,21 +566,22 @@ class NoseTestSuiteRunner(BasicNoseRunner):
                         style, connection)
                 else:
                     reset_statements = connection.ops.sequence_reset_sql(
-                            style, self._get_models_for_connection(connection))
+                        style, self._get_models_for_connection(connection))
 
-                for reset_statement in reset_statements:
-                    cursor.execute(reset_statement)
-
-                # Django v1.3 (https://code.djangoproject.com/ticket/9964)
-                # starts using commit_unless_managed() for individual
-                # connections. Backwards compatibility for Django 1.2 is to use
-                # the generic transaction function.
-                transaction.commit_unless_managed(using=connection.alias)
+                if hasattr(transaction, "atomic"):
+                    with transaction.atomic(using=connection.alias):
+                        for reset_statement in reset_statements:
+                            cursor.execute(reset_statement)
+                else:
+                    # Django < 1.6
+                    for reset_statement in reset_statements:
+                        cursor.execute(reset_statement)
+                    transaction.commit_unless_managed(using=connection.alias)
 
                 # Each connection has its own creation object, so this affects
                 # only a single connection:
                 creation.create_test_db = MethodType(
-                        _skip_create_test_db, creation)
+                    _skip_create_test_db, creation)
 
         Command.handle = _foreign_key_ignoring_handle
 
@@ -538,5 +593,5 @@ class NoseTestSuiteRunner(BasicNoseRunner):
         """Leave those poor, reusable databases alone if REUSE_DB is true."""
         if not _reusing_db():
             return super(NoseTestSuiteRunner, self).teardown_databases(
-                    *args, **kwargs)
+                *args, **kwargs)
         # else skip tearing down the DB so we can reuse it next time
